/* Source code derived from UNIX Network Programming Vol1, 
   Third Edition: The Sockets Networking API 
 */

ssize_t                         /* Write "n" bytes to a descriptor. */
writen(int fd, const void *vptr, size_t n)
{
     size_t nleft;
     ssize_t nwritten;
     const char *ptr;

     ptr = vptr;
     nleft = n;
     while (nleft > 0) {
	  if ( (nwritten = write(fd, ptr, nleft)) <= 0) {
	       if (nwritten < 0 && errno == EINTR)
		    nwritten = 0;   /* and call write()
							 * again */
	       else {
		    printf("returning error within writen\n");
		    return (-1);    /* error */
	       }
	  }

	  nleft -= nwritten;
	  ptr += nwritten;
     }
     return (n);
}

ssize_t                         /* Read "n" bytes from a descriptor. */
readn(int fd, void *vptr, size_t n)
{
     size_t  nleft;
     ssize_t nread;
     char   *ptr;

     ptr = vptr;
     nleft = n;
     while (nleft > 0) {
	  if ( (nread = read(fd, ptr, nleft)) < 0) {
	       if (errno == EINTR)
		    nread = 0;      /* and call read() again */
	       else
		    return (-1);
	  } else if (nread == 0)
	       break;              /* EOF */

	  nleft -= nread;
	  ptr += nread;
     }
     return (n - nleft);         /* return >= 0 */
}
